{% extends "webgallery/base/base.html" %}

{% block title %}
    {{ dataset.getName }}
{% endblock %}


{% block body %}

    {% if dataset.getParent %}
    {% with p=dataset.getParent %}
        Back to: <a href="{% url 'webgallery_show_project' p.id %}">{{ p.getName }}</a>
    {% endwith %}
    {% endif %}

    <h2>{{ dataset.getName }}</h2>

    <p>{{ dataset.getDescription }}</p>

    <hr>

    {% for i in dataset.listChildren %}
        <a href="{% url 'webgateway_full_viewer' i.id %}">
            <img title="{{ i.getName }}" src="{% url 'webgateway_render_thumbnail' i.id 96 %}" />
        </a>
    {% endfor %}

{% endblock %}