.super_categories {
  margin-bottom: 20px;
}
.super_categories img {
  margin-bottom: 0;
}
.search-by {
  margin-top: 5px;
}
.category {
  width:100%;
  overflow:auto;
  background: white;
  margin-bottom: 30px;
}
.category>div {
  display: inline-flex;
}

.study {
  width: 250px;
  float: left;
  position: relative;
}
.row .study {
  flex-shrink: 0;
  margin: 10px;
  margin-top: 5px;
}
.study p {
  margin-bottom: 0;
}
.study a {
  font-weight: inherit;
}
.study a:hover .studyText{
  color: #92cef8;
}
a.pubmed {
  color: #44a6ea
}
.img_overlay {
  position: absolute;
  top: 50%;
  left: 50%;
  transform: translate(-50%, -50%);
}
.overlay_text {
  background: #ff9800;
  padding: 10px;
  color: white;
  border-radius: 2px;
}
.studyBackground {
  position: absolute;
  width: 100%;
  height: 100%;
  text-align: center;
  overflow: hidden;
}
.studyBackground, .studyText {
  border-radius: 10px;
}
.studyText {
  position: absolute;
  width: 100%;
  color: white;
  background: rgba(0,0,0,0.3);
  font-size: 90%;
  bottom: 0;
  padding: 5px;
  border-top-left-radius: 0;
  border-top-right-radius: 0;
}
.studyAuthors {
  position: absolute;
  top: 0;
  font-size: 80%;
  width: 200%;
  text-align: left;
  visibility: hidden;
}
.study:hover .studyAuthors {
  /*visibility: visible;*/
}
.viewerLink {
  display: block;
  position: absolute;
  top: 5px;
  right: 5px;
}
.study a.viewerLink {
  color: white;
  visibility: hidden;
}
.study:hover a.viewerLink {
  visibility: visible;
}

maprText {
  width: 100%
}

#spinner {
  width: 20px;
  color: #bbb;
  position: absolute;
  top: 10px;
  right: 25px;
  visibility: hidden;
}

#filterSpinner {
  display: none;
}
#filterSpinner svg {
  width: 50px;
  margin: 20px;
  color: #bbb;
}

/* Fix issues from combining foundation css with jquery-ui auto-complete */
.ui-autocomplete a {
  font-weight: normal;
}
.ui-autocomplete {
  padding-left: 20px;
  text-indent: -10px;
}

#studies.studiesLayout {
  width: 86%;
  margin-left: auto;
  margin-right: auto;
}

#studies .studiesLayout {
  width: 79%;
  margin-left: auto;
  margin-right: auto;
}

.studyImage {
  position: relative;
  height: 200px;
  width: 250px;
  background: no-repeat center;
  background-size: cover;
  position: relative;
  border-radius: 10px;
}

/* Always show scrollbars https://gist.github.com/IceCreamYou/cd517596e5847a88e2bb0a091da43fb4 */
::-webkit-scrollbar-track:vertical {
    border-left: 1px solid #E7E7E7;
    box-shadow: 1px 0 1px 0 #F6F6F6 inset, -1px 0 1px 0 #F6F6F6 inset;
}

::-webkit-scrollbar-track:horizontal {
    border-top: 1px solid #E7E7E7;
    box-shadow: 0 1px 1px 0 #F6F6F6 inset, 0 -1px 1px 0 #F6F6F6 inset;
}

::-webkit-scrollbar {
    -webkit-appearance: none;
    background-color: #FAFAFA;
    width: 16px;
}

::-webkit-scrollbar-thumb {
    background-clip: padding-box;
    background-color: #C1C1C1;
    border-color: transparent;
    border-radius: 9px 8px 8px 9px;
    border-style: solid;
    border-width: 3px 3px 3px 4px; /* Workaround because margins aren't supported */
    box-shadow: 0 0 1px rgba(255, 255, 255, 0.5);
}

::-webkit-scrollbar-thumb:hover {
    background-color: rgba(0, 0, 0, 0.5);
}
