<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="utf-8">
    <meta http-equiv="X-UA-Compatible" content="IE=edge">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <meta name="description" content="">
    <meta name="author" content="">
    <link rel="shortcut icon" href="../../assets/ico/favicon.ico">

    <title>{% block title %}{% endblock %}</title>

    <!-- Bootstrap core CSS -->
    <link href="{% static 'gallery/3rdparty/bootstrap-3.1.1/css/bootstrap.min.css' %}" rel="stylesheet">

    <!-- Custom styles for this template -->
    <style>
        body {
          padding-top: 50px;
          padding-bottom: 20px;
        }
    </style>

    <!-- Just for debugging purposes. Don't actually copy this line! -->
    <!--[if lt IE 9]><script src="../../assets/js/ie8-responsive-file-warning.js"></script><![endif]-->

    <!-- HTML5 shim and Respond.js IE8 support of HTML5 elements and media queries -->
    <!--[if lt IE 9]>
      <script src="https://oss.maxcdn.com/libs/html5shiv/3.7.0/html5shiv.js"></script>
      <script src="https://oss.maxcdn.com/libs/respond.js/1.4.2/respond.min.js"></script>
    <![endif]-->
  </head>

  <body>

    <div class="navbar navbar-inverse navbar-fixed-top" role="navigation">
      <div class="container">
        <div class="navbar-header">
          <button type="button" class="navbar-toggle" data-toggle="collapse" data-target=".navbar-collapse">
            <span class="sr-only">Toggle navigation</span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
            <span class="icon-bar"></span>
          </button>
          <a class="navbar-brand" href="{% url 'webgallery_index' %}">OMERO.gallery</a>
        </div>
        <div class="navbar-collapse collapse pull-right">
            <!-- <form class="navbar-form navbar-right" role="form"> -->
                {% block navbar %}
                <ul class="nav navbar-nav">
            <li>
                <a href="#">Logged in as: {{ ome.user.getFullName }}</a></li>
            <li><a href="{% url 'weblogout' %}">Logout</a></li>
          </ul>
            {% endblock %}
        </div><!--/.navbar-collapse -->
      </div>
    </div>

    {% block jumbotron %}
    {% endblock %}

    <div class="container">
      <!-- Example row of columns -->
      {% block body %}
        {% endblock %}

      <hr>

      <footer>
        <p>&copy; Company 2014</p>
      </footer>
    </div> <!-- /container -->


    <!-- Bootstrap core JavaScript
    ================================================== -->
    <!-- Placed at the end of the document so the pages load faster -->
    <script src="https://ajax.googleapis.com/ajax/libs/jquery/1.11.0/jquery.min.js"></script>
    <script src="{% static 'gallery/3rdparty/bootstrap-3.1.1/js/bootstrap.min.js' %}"></script>
  </body>
</html>
